@@ -19,4 +19,8 @@
 from .client import Client
 from .ext import BaseClient, ChatAction, Emoji, ParseMode
 from .filters import Filters
-from .handlers import MessageHandler, CallbackQueryHandler, RawUpdateHandler
+from .handlers import (
+    MessageHandler, DeletedMessagesHandler,
+    CallbackQueryHandler, RawUpdateHandler,
+    DisconnectHandler, UserStatusHandler
+)
