@@ -33,6 +33,8 @@ import time
 from configparser import ConfigParser
 from datetime import datetime
 from hashlib import sha256, md5
+from importlib import import_module
+from pathlib import Path
 from signal import signal, SIGINT, SIGTERM, SIGABRT
 from threading import Thread
 
@@ -43,8 +45,9 @@ from pyrogram.api.errors import (
     PhoneNumberUnoccupied, PhoneCodeInvalid, PhoneCodeHashEmpty,
     PhoneCodeExpired, PhoneCodeEmpty, SessionPasswordNeeded,
     PasswordHashInvalid, FloodWait, PeerIdInvalid, FirstnameInvalid, PhoneNumberBanned,
-    VolumeLocNotFound, UserMigrate, FileIdInvalid)
+    VolumeLocNotFound, UserMigrate, FileIdInvalid, ChannelPrivate)
 from pyrogram.client.handlers import DisconnectHandler
+from pyrogram.client.handlers.handler import Handler
 from pyrogram.crypto import AES
 from pyrogram.session import Auth, Session
 from .dispatcher import Dispatcher
@@ -90,6 +93,10 @@ class Client(Methods, BaseClient):
             Code of the language used on the client, in ISO 639-1 standard. Defaults to "en".
             This is an alternative way to set it if you don't want to use the *config.ini* file.
 
+        ipv6 (``bool``, *optional*):
+            Pass True to connect to Telegram using IPv6.
+            Defaults to False (IPv4).
+
         proxy (``dict``, *optional*):
             Your SOCKS5 Proxy settings as dict,
             e.g.: *dict(hostname="11.22.33.44", port=1080, username="user", password="pass")*.
@@ -136,6 +143,11 @@ class Client(Methods, BaseClient):
 
         config_file (``str``, *optional*):
             Path of the configuration file. Defaults to ./config.ini
+
+        plugins_dir (``str``, *optional*):
+            Define a custom directory for your plugins. The plugins directory is the location in your
+            filesystem where Pyrogram will automatically load your update handlers.
+            Defaults to None (plugins disabled).
     """
 
     def __init__(self,
@@ -146,6 +158,7 @@ class Client(Methods, BaseClient):
                  device_model: str = None,
                  system_version: str = None,
                  lang_code: str = None,
+                 ipv6: bool = False,
                  proxy: dict = None,
                  test_mode: bool = False,
                  phone_number: str = None,
@@ -154,9 +167,10 @@ class Client(Methods, BaseClient):
                  force_sms: bool = False,
                  first_name: str = None,
                  last_name: str = None,
-                 workers: int = 4,
-                 workdir: str = ".",
-                 config_file: str = "./config.ini"):
+                 workers: int = BaseClient.WORKERS,
+                 workdir: str = BaseClient.WORKDIR,
+                 config_file: str = BaseClient.CONFIG_FILE,
+                 plugins_dir: str = None):
         super().__init__()
 
         self.session_name = session_name
@@ -166,6 +180,7 @@ class Client(Methods, BaseClient):
         self.device_model = device_model
         self.system_version = system_version
         self.lang_code = lang_code
+        self.ipv6 = ipv6
         # TODO: Make code consistent, use underscore for private/protected fields
         self._proxy = proxy
         self.test_mode = test_mode
@@ -178,9 +193,17 @@ class Client(Methods, BaseClient):
         self.workers = workers
         self.workdir = workdir
         self.config_file = config_file
+        self.plugins_dir = plugins_dir
 
         self.dispatcher = Dispatcher(self, workers)
 
+    def __enter__(self):
+        self.start()
+        return self
+
+    def __exit__(self, exc_type, exc_val, exc_tb):
+        self.stop()
+
     @property
     def proxy(self):
         return self._proxy
@@ -195,17 +218,19 @@ class Client(Methods, BaseClient):
         Requires no parameters.
 
         Raises:
-            :class:`Error <pyrogram.Error>`
+            :class:`Error <pyrogram.Error>` in case of a Telegram RPC error.
+            ``ConnectionError`` in case you try to start an already started Client.
         """
         if self.is_started:
             raise ConnectionError("Client has already been started")
 
         if self.BOT_TOKEN_RE.match(self.session_name):
-            self.token = self.session_name
+            self.bot_token = self.session_name
             self.session_name = self.session_name.split(":")[0]
 
         self.load_config()
         self.load_session()
+        self.load_plugins()
 
         self.session = Session(
             self,
@@ -216,28 +241,33 @@ class Client(Methods, BaseClient):
         self.session.start()
         self.is_started = True
 
-        if self.user_id is None:
-            if self.token is None:
-                self.authorize_user()
+        try:
+            if self.user_id is None:
+                if self.bot_token is None:
+                    self.authorize_user()
+                else:
+                    self.authorize_bot()
+
+                self.save_session()
+
+            if self.bot_token is None:
+                now = time.time()
+
+                if abs(now - self.date) > Client.OFFLINE_SLEEP:
+                    self.peers_by_username = {}
+                    self.peers_by_phone = {}
+
+                    self.get_initial_dialogs()
+                    self.get_contacts()
+                else:
+                    self.send(functions.messages.GetPinnedDialogs())
+                    self.get_initial_dialogs_chunk()
             else:
-                self.authorize_bot()
-
-            self.save_session()
-
-        if self.token is None:
-            now = time.time()
-
-            if abs(now - self.date) > Client.OFFLINE_SLEEP:
-                self.peers_by_username = {}
-                self.peers_by_phone = {}
-
-                self.get_dialogs()
-                self.get_contacts()
-            else:
-                self.send(functions.messages.GetPinnedDialogs())
-                self.get_dialogs_chunk(0)
-        else:
-            self.send(functions.updates.GetState())
+                self.send(functions.updates.GetState())
+        except Exception as e:
+            self.is_started = False
+            self.session.stop()
+            raise e
 
         for i in range(self.UPDATES_WORKERS):
             self.updates_workers_list.append(
@@ -267,6 +297,9 @@ class Client(Methods, BaseClient):
     def stop(self):
         """Use this method to manually stop the Client.
         Requires no parameters.
+
+        Raises:
+            ``ConnectionError`` in case you try to stop an already stopped Client.
         """
         if not self.is_started:
             raise ConnectionError("Client is already stopped")
@@ -326,7 +359,7 @@ class Client(Methods, BaseClient):
         Requires no parameters.
 
         Raises:
-            :class:`Error <pyrogram.Error>`
+            :class:`Error <pyrogram.Error>` in case of a Telegram RPC error.
         """
         self.start()
         self.idle()
@@ -381,14 +414,14 @@ class Client(Methods, BaseClient):
                     flags=0,
                     api_id=self.api_id,
                     api_hash=self.api_hash,
-                    bot_auth_token=self.token
+                    bot_auth_token=self.bot_token
                 )
             )
         except UserMigrate as e:
             self.session.stop()
 
             self.dc_id = e.x
-            self.auth_key = Auth(self.dc_id, self.test_mode, self._proxy).create()
+            self.auth_key = Auth(self.dc_id, self.test_mode, self.ipv6, self._proxy).create()
 
             self.session = Session(
                 self,
@@ -433,7 +466,7 @@ class Client(Methods, BaseClient):
                 self.session.stop()
 
                 self.dc_id = e.x
-                self.auth_key = Auth(self.dc_id, self.test_mode, self._proxy).create()
+                self.auth_key = Auth(self.dc_id, self.test_mode, self.ipv6, self._proxy).create()
 
                 self.session = Session(
                     self,
@@ -613,7 +646,7 @@ class Client(Methods, BaseClient):
                 if phone is not None:
                     self.peers_by_phone[phone] = input_peer
 
-            if isinstance(entity, types.Chat):
+            if isinstance(entity, (types.Chat, types.ChatForbidden)):
                 chat_id = entity.id
                 peer_id = -chat_id
 
@@ -623,7 +656,7 @@ class Client(Methods, BaseClient):
 
                 self.peers_by_id[peer_id] = input_peer
 
-            if isinstance(entity, types.Channel):
+            if isinstance(entity, (types.Channel, types.ChannelForbidden)):
                 channel_id = entity.id
                 peer_id = int("-100" + str(channel_id))
 
@@ -632,7 +665,7 @@ class Client(Methods, BaseClient):
                 if access_hash is None:
                     continue
 
-                username = entity.username
+                username = getattr(entity, "username", None)
 
                 input_peer = types.InputPeerChannel(
                     channel_id=channel_id,
@@ -711,7 +744,9 @@ class Client(Methods, BaseClient):
 
                     file_name = "{}_{}_{}{}".format(
                         media_type_str,
-                        datetime.fromtimestamp(media.date or time.time()).strftime("%Y-%m-%d_%H-%M-%S"),
+                        datetime.fromtimestamp(
+                            getattr(media, "date", None) or time.time()
+                        ).strftime("%Y-%m-%d_%H-%M-%S"),
                         self.rnd_id(),
                         extension
                     )
@@ -776,27 +811,33 @@ class Client(Methods, BaseClient):
                         pts = getattr(update, "pts", None)
                         pts_count = getattr(update, "pts_count", None)
 
+                        if isinstance(update, types.UpdateChannelTooLong):
+                            log.warning(update)
+
                         if isinstance(update, types.UpdateNewChannelMessage):
                             message = update.message
 
                             if not isinstance(message, types.MessageEmpty):
-                                diff = self.send(
-                                    functions.updates.GetChannelDifference(
-                                        channel=self.resolve_peer(int("-100" + str(channel_id))),
-                                        filter=types.ChannelMessagesFilter(
-                                            ranges=[types.MessageRange(
-                                                min_id=update.message.id,
-                                                max_id=update.message.id
-                                            )]
-                                        ),
-                                        pts=pts - pts_count,
-                                        limit=pts
+                                try:
+                                    diff = self.send(
+                                        functions.updates.GetChannelDifference(
+                                            channel=self.resolve_peer(int("-100" + str(channel_id))),
+                                            filter=types.ChannelMessagesFilter(
+                                                ranges=[types.MessageRange(
+                                                    min_id=update.message.id,
+                                                    max_id=update.message.id
+                                                )]
+                                            ),
+                                            pts=pts - pts_count,
+                                            limit=pts
+                                        )
                                     )
-                                )
-
-                                if not isinstance(diff, types.updates.ChannelDifferenceEmpty):
-                                    updates.users += diff.users
-                                    updates.chats += diff.chats
+                                except ChannelPrivate:
+                                    pass
+                                else:
+                                    if not isinstance(diff, types.updates.ChannelDifferenceEmpty):
+                                        updates.users += diff.users
+                                        updates.chats += diff.chats
 
                         if channel_id and pts:
                             if channel_id not in self.channels_pts:
@@ -834,6 +875,8 @@ class Client(Methods, BaseClient):
                         self.dispatcher.updates.put((diff.other_updates[0], [], []))
                 elif isinstance(updates, types.UpdateShort):
                     self.dispatcher.updates.put((updates.update, [], []))
+                elif isinstance(updates, types.UpdatesTooLong):
+                    log.warning(updates)
             except Exception as e:
                 log.error(e, exc_info=True)
 
@@ -848,7 +891,7 @@ class Client(Methods, BaseClient):
 
         Args:
             data (``Object``):
-                The API Scheme function filled with proper arguments.
+                The API Schema function filled with proper arguments.
 
             retries (``int``):
                 Number of retries.
@@ -857,7 +900,7 @@ class Client(Methods, BaseClient):
                 Timeout in seconds.
 
         Raises:
-            :class:`Error <pyrogram.Error>`
+            :class:`Error <pyrogram.Error>` in case of a Telegram RPC error.
         """
         if not self.is_started:
             raise ConnectionError("Client has not been started")
@@ -885,30 +928,18 @@ class Client(Methods, BaseClient):
                     "More info: https://docs.pyrogram.ml/start/ProjectSetup#configuration"
                 )
 
-        for option in {"app_version", "device_model", "system_version", "lang_code"}:
+        for option in ["app_version", "device_model", "system_version", "lang_code"]:
             if getattr(self, option):
                 pass
             else:
-                setattr(self, option, Client.APP_VERSION)
-
                 if parser.has_section("pyrogram"):
                     setattr(self, option, parser.get(
                         "pyrogram",
                         option,
                         fallback=getattr(Client, option.upper())
                     ))
-
-        if self.lang_code:
-            pass
-        else:
-            self.lang_code = Client.LANG_CODE
-
-            if parser.has_section("pyrogram"):
-                self.lang_code = parser.get(
-                    "pyrogram",
-                    "lang_code",
-                    fallback=Client.LANG_CODE
-                )
+                else:
+                    setattr(self, option, getattr(Client, option.upper()))
 
         if self._proxy:
             self._proxy["enabled"] = True
@@ -929,7 +960,7 @@ class Client(Methods, BaseClient):
         except FileNotFoundError:
             self.dc_id = 1
             self.date = 0
-            self.auth_key = Auth(self.dc_id, self.test_mode, self._proxy).create()
+            self.auth_key = Auth(self.dc_id, self.test_mode, self.ipv6, self._proxy).create()
         else:
             self.dc_id = s["dc_id"]
             self.test_mode = s["test_mode"]
@@ -952,6 +983,45 @@ class Client(Methods, BaseClient):
                 if peer:
                     self.peers_by_phone[k] = peer
 
+    def load_plugins(self):
+        if self.plugins_dir is not None:
+            plugins_count = 0
+
+            for path in Path(self.plugins_dir).rglob("*.py"):
+                file_path = os.path.splitext(str(path))[0]
+                import_path = []
+
+                while file_path:
+                    file_path, tail = os.path.split(file_path)
+                    import_path.insert(0, tail)
+
+                import_path = ".".join(import_path)
+                module = import_module(import_path)
+
+                for name in dir(module):
+                    # noinspection PyBroadException
+                    try:
+                        handler, group = getattr(module, name)
+
+                        if isinstance(handler, Handler) and isinstance(group, int):
+                            self.add_handler(handler, group)
+
+                            log.info('{}("{}") from "{}" loaded in group {}'.format(
+                                type(handler).__name__, name, import_path, group))
+
+                            plugins_count += 1
+                    except Exception:
+                        pass
+
+            if plugins_count > 0:
+                log.warning('Successfully loaded {} plugin{} from "{}"'.format(
+                    plugins_count,
+                    "s" if plugins_count > 1 else "",
+                    self.plugins_dir
+                ))
+            else:
+                log.warning('No plugin loaded: "{}" doesn\'t contain any valid plugin'.format(self.plugins_dir))
+
     def save_session(self):
         auth_key = base64.b64encode(self.auth_key).decode()
         auth_key = [auth_key[i: i + 43] for i in range(0, len(auth_key), 43)]
@@ -971,13 +1041,17 @@ class Client(Methods, BaseClient):
                 indent=4
             )
 
-    def get_dialogs_chunk(self, offset_date):
+    def get_initial_dialogs_chunk(self, offset_date: int = 0):
         while True:
             try:
                 r = self.send(
                     functions.messages.GetDialogs(
-                        offset_date, 0, types.InputPeerEmpty(),
-                        self.DIALOGS_AT_ONCE, True
+                        offset_date=offset_date,
+                        offset_id=0,
+                        offset_peer=types.InputPeerEmpty(),
+                        limit=self.DIALOGS_AT_ONCE,
+                        hash=0,
+                        exclude_pinned=True
                     )
                 )
             except FloodWait as e:
@@ -987,74 +1061,56 @@ class Client(Methods, BaseClient):
                 log.info("Total peers: {}".format(len(self.peers_by_id)))
                 return r
 
-    def get_dialogs(self):
+    def get_initial_dialogs(self):
         self.send(functions.messages.GetPinnedDialogs())
 
-        dialogs = self.get_dialogs_chunk(0)
+        dialogs = self.get_initial_dialogs_chunk()
         offset_date = utils.get_offset_date(dialogs)
 
         while len(dialogs.dialogs) == self.DIALOGS_AT_ONCE:
-            dialogs = self.get_dialogs_chunk(offset_date)
+            dialogs = self.get_initial_dialogs_chunk(offset_date)
             offset_date = utils.get_offset_date(dialogs)
 
-        self.get_dialogs_chunk(0)
+        self.get_initial_dialogs_chunk()
 
     def resolve_peer(self, peer_id: int or str):
-        """Use this method to get the *InputPeer* of a known *peer_id*.
+        """Use this method to get the InputPeer of a known peer_id.
 
-        It is intended to be used when working with Raw Functions (i.e: a Telegram API method you wish to use which is
-        not available yet in the Client class as an easy-to-use method).
+        This is a utility method intended to be used only when working with Raw Functions (i.e: a Telegram API method
+        you wish to use which is not available yet in the Client class as an easy-to-use method), whenever an InputPeer
+        type is required.
 
         Args:
-            peer_id (``int`` | ``str`` | ``Peer``):
-                The Peer ID you want to extract the InputPeer from. Can be one of these types: ``int`` (direct ID),
-                ``str`` (@username), :obj:`PeerUser <pyrogram.api.types.PeerUser>`,
-                :obj:`PeerChat <pyrogram.api.types.PeerChat>`, :obj:`PeerChannel <pyrogram.api.types.PeerChannel>`
+            peer_id (``int`` | ``str``):
+                The peer id you want to extract the InputPeer from.
+                Can be a direct id (int), a username (str) or a phone number (str).
 
         Returns:
-            :obj:`InputPeerUser <pyrogram.api.types.InputPeerUser>` or
-            :obj:`InputPeerChat <pyrogram.api.types.InputPeerChat>` or
-            :obj:`InputPeerChannel <pyrogram.api.types.InputPeerChannel>` depending on the *peer_id*.
+            On success, the resolved peer id is returned in form of an InputPeer object.
 
         Raises:
-            :class:`Error <pyrogram.Error>`
+            :class:`Error <pyrogram.Error>` in case of a Telegram RPC error.
+            ``KeyError`` in case the peer doesn't exist in the internal database.
         """
         if type(peer_id) is str:
             if peer_id in ("self", "me"):
                 return types.InputPeerSelf()
 
-            match = self.INVITE_LINK_RE.match(peer_id)
-
-            try:
-                decoded = base64.b64decode(match.group(1) + "=" * (-len(match.group(1)) % 4), "-_")
-                return self.resolve_peer(struct.unpack(">2iq", decoded)[1])
-            except (AttributeError, binascii.Error, struct.error):
-                pass
-
             peer_id = re.sub(r"[@+\s]", "", peer_id.lower())
 
             try:
                 int(peer_id)
             except ValueError:
-                try:
-                    return self.peers_by_username[peer_id]
-                except KeyError:
+                if peer_id not in self.peers_by_username:
                     self.send(functions.contacts.ResolveUsername(peer_id))
-                    return self.peers_by_username[peer_id]
+
+                return self.peers_by_username[peer_id]
             else:
                 try:
                     return self.peers_by_phone[peer_id]
                 except KeyError:
                     raise PeerIdInvalid
 
-        if type(peer_id) is not int:
-            if isinstance(peer_id, types.PeerUser):
-                peer_id = peer_id.user_id
-            elif isinstance(peer_id, types.PeerChat):
-                peer_id = -peer_id.chat_id
-            elif isinstance(peer_id, types.PeerChannel):
-                peer_id = int("-100" + str(peer_id.channel_id))
-
         try:  # User
             return self.peers_by_id[peer_id]
         except KeyError:
@@ -1074,6 +1130,13 @@ class Client(Methods, BaseClient):
                   progress_args: tuple = ()):
         part_size = 512 * 1024
         file_size = os.path.getsize(path)
+        
+        if file_size == 0:
+            raise ValueError("File size equals to 0 B")
+        
+        if file_size > 1500 * 1024 * 1024:
+            raise ValueError("Telegram doesn't support uploading files bigger than 1500 MiB")
+            
         file_total_parts = int(math.ceil(file_size / part_size))
         is_big = True if file_size > 10 * 1024 * 1024 else False
         is_missing_part = True if file_id is not None else False
@@ -1166,7 +1229,7 @@ class Client(Methods, BaseClient):
                     session = Session(
                         self,
                         dc_id,
-                        Auth(dc_id, self.test_mode, self._proxy).create(),
+                        Auth(dc_id, self.test_mode, self.ipv6, self._proxy).create(),
                         is_media=True
                     )
 
@@ -1229,8 +1292,6 @@ class Client(Methods, BaseClient):
                             break
 
                         f.write(chunk)
-                        f.flush()
-                        os.fsync(f.fileno())
 
                         offset += limit
 
@@ -1253,7 +1314,7 @@ class Client(Methods, BaseClient):
                         cdn_session = Session(
                             self,
                             r.dc_id,
-                            Auth(r.dc_id, self.test_mode, self._proxy).create(),
+                            Auth(r.dc_id, self.test_mode, self.ipv6, self._proxy).create(),
                             is_media=True,
                             is_cdn=True
                         )
@@ -1313,8 +1374,6 @@ class Client(Methods, BaseClient):
                                 assert h.hash == sha256(cdn_chunk).digest(), "Invalid CDN hash part {}".format(i)
 
                             f.write(decrypted_chunk)
-                            f.flush()
-                            os.fsync(f.fileno())
 
                             offset += limit
 
