@@ -16,31 +16,25 @@
 # You should have received a copy of the GNU Lesser General Public License
 # along with Pyrogram.  If not, see <http://www.gnu.org/licenses/>.
 
-from .audio import Audio
-from .callback_query import CallbackQuery
-from .chat import Chat
-from .chat_member import ChatMember
-from .chat_photo import ChatPhoto
-from .contact import Contact
-from .document import Document
-from .gif import GIF
-from .input_media_photo import InputMediaPhoto
-from .input_media_video import InputMediaVideo
-from .input_phone_contact import InputPhoneContact
-from .location import Location
-from .message import Message
-from .message_entity import MessageEntity
-from .messages import Messages
-from .photo_size import PhotoSize
-from .reply_markup import (
+from .bots import (
+    CallbackQuery, ForceReply, InlineKeyboardButton, InlineKeyboardMarkup,
+    KeyboardButton, ReplyKeyboardMarkup, ReplyKeyboardRemove
+)
+from .bots import (
     ForceReply, InlineKeyboardButton, InlineKeyboardMarkup,
     KeyboardButton, ReplyKeyboardMarkup, ReplyKeyboardRemove
 )
-from .sticker import Sticker
+from .input_media import (
+    InputMediaAudio, InputPhoneContact, InputMediaVideo, InputMediaPhoto,
+    InputMediaDocument, InputMediaAnimation
+)
+from .messages_and_media import (
+    Audio, Contact, Document, Animation, Location, Photo, PhotoSize,
+    Sticker, Venue, Video, VideoNote, Voice, UserProfilePhotos,
+    Message, Messages, MessageEntity
+)
 from .update import Update
-from .user import User
-from .user_profile_photos import UserProfilePhotos
-from .venue import Venue
-from .video import Video
-from .video_note import VideoNote
-from .voice import Voice
+from .user_and_chats import (
+    Chat, ChatMember, ChatMembers, ChatPhoto,
+    Dialog, Dialogs, User, UserStatus
+)
